# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_object
----------------------------------

Functional tests for `shade` object methods.
"""

import tempfile

from testtools import content

from shade import openstack_cloud
from shade.tests import base


class TestObject(base.TestCase):

    def setUp(self):
        super(TestObject, self).setUp()
        self.cloud = openstack_cloud(cloud='devstack')
        if not self.cloud.has_service('object-store'):
            self.skipTest('Object service not supported by cloud')

    def test_create_object(self):
        '''Test uploading small and large files.'''
        container_name = self.getUniqueString('container')
        self.addDetail('container', content.text_content(container_name))
        self.addCleanup(self.cloud.delete_container, container_name)
        self.cloud.create_container(container_name)
        self.assertEqual(container_name,
                         self.cloud.list_containers()[0]['name'])
        sizes = (
            (64 * 1024, 1),      # 64K, one segment
            (50 * 1024 ** 2, 5)  # 50MB, 5 segments
        )
        for size, nseg in sizes:
            segment_size = round(size / nseg)
            with tempfile.NamedTemporaryFile() as sparse_file:
                sparse_file.seek(size)
                sparse_file.write("\0")
                sparse_file.flush()
                name = 'test-%d' % size
                self.cloud.create_object(container_name, name,
                                         sparse_file.name,
                                         segment_size=segment_size)
                self.assertFalse(self.cloud.is_object_stale(
                    container_name, name,
                    sparse_file.name
                    )
                )
            self.assertIsNotNone(
                self.cloud.get_object_metadata(container_name, name))
            self.assertIsNotNone(
                self.cloud.get_object(container_name, name))
            self.assertEqual(
                name,
                self.cloud.list_objects(container_name)[0]['name'])
            self.cloud.delete_object(container_name, name)
        self.assertEqual([], self.cloud.list_objects(container_name))
        self.assertEqual(container_name,
                         self.cloud.list_containers()[0]['name'])
        self.cloud.delete_container(container_name)



# -*- coding: utf-8 -*-
# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import os_client_config
from os_client_config import cloud_config
from swiftclient import service as swift_service
from swiftclient import exceptions as swift_exc
import testtools

import shade
import shade.openstackcloud
from shade import exc
from shade import OpenStackCloud
from shade.tests.unit import base


class TestObject(base.TestCase):

    def setUp(self):
        super(TestObject, self).setUp()
        config = os_client_config.OpenStackConfig()
        self.cloud = OpenStackCloud(
            cloud_config=config.get_one_cloud(validate=False))

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_swift_client_no_endpoint(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = None
        get_session_mock.return_value = session_mock
        e = self.assertRaises(
            exc.OpenStackCloudException, lambda: self.cloud.swift_client)
        self.assertIn(
            'Failed to instantiate object-store client.', str(e))

    @mock.patch.object(shade.OpenStackCloud, 'auth_token')
    @mock.patch.object(shade.OpenStackCloud, 'get_session_endpoint')
    def test_swift_service(self, endpoint_mock, auth_mock):
        endpoint_mock.return_value = 'slayer'
        auth_mock.return_value = 'zulu'
        self.assertIsInstance(self.cloud.swift_service,
                              swift_service.SwiftService)
        endpoint_mock.assert_called_with(service_key='object-store')

    @mock.patch.object(shade.OpenStackCloud, 'get_session_endpoint')
    def test_swift_service_no_endpoint(self, endpoint_mock):
        endpoint_mock.side_effect = KeyError
        e = self.assertRaises(exc.OpenStackCloudException, lambda:
                              self.cloud.swift_service)
        self.assertIn(
            'Error constructing swift client', str(e))

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_object_segment_size(self, swift_mock):
        swift_mock.get_capabilities.return_value = {'swift':
                                                    {'max_file_size': 1000}}
        self.assertEqual(900, self.cloud.get_object_segment_size(900))
        self.assertEqual(1000, self.cloud.get_object_segment_size(1000))
        self.assertEqual(1000, self.cloud.get_object_segment_size(1100))

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_object_segment_size_http_412(self, swift_mock):
        swift_mock.get_capabilities.side_effect = swift_exc.ClientException(
            "Precondition failed", http_status=412)
        self.assertEqual(shade.openstackcloud.DEFAULT_OBJECT_SEGMENT_SIZE,
                         self.cloud.get_object_segment_size(None))

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_create_container(self, mock_swift):
        """Test creating a (private) container"""
        name = 'test_container'
        mock_swift.head_container.return_value = None

        self.cloud.create_container(name)

        expected_head_container_calls = [
            # once for exist test
            mock.call(container=name),
            # once for the final return
            mock.call(container=name, skip_cache=True)
        ]
        self.assertTrue(expected_head_container_calls,
                        mock_swift.head_container.call_args_list)
        mock_swift.put_container.assert_called_once_with(container=name)
        # Because the default is 'private', we shouldn't be calling update
        self.assertFalse(mock_swift.post_container.called)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_create_container_public(self, mock_swift):
        """Test creating a public container"""
        name = 'test_container'
        mock_swift.head_container.return_value = None

        self.cloud.create_container(name, public=True)

        expected_head_container_calls = [
            # once for exist test
            mock.call(container=name),
            # once for the final return
            mock.call(container=name, skip_cache=True)
        ]
        self.assertTrue(expected_head_container_calls,
                        mock_swift.head_container.call_args_list)
        mock_swift.put_container.assert_called_once_with(container=name)
        mock_swift.post_container.assert_called_once_with(
            container=name,
            headers={'x-container-read':
                     shade.openstackcloud.OBJECT_CONTAINER_ACLS['public']}
        )

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_create_container_exists(self, mock_swift):
        """Test creating a container that already exists"""
        name = 'test_container'
        fake_container = dict(id='1', name='name')
        mock_swift.head_container.return_value = fake_container
        container = self.cloud.create_container(name)
        mock_swift.head_container.assert_called_once_with(container=name)
        self.assertEqual(fake_container, container)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_delete_container(self, mock_swift):
        name = 'test_container'
        self.cloud.delete_container(name)
        mock_swift.delete_container.assert_called_once_with(container=name)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_delete_container_404(self, mock_swift):
        """No exception when deleting a container that does not exist"""
        name = 'test_container'
        mock_swift.delete_container.side_effect = swift_exc.ClientException(
            'ERROR', http_status=404)
        self.cloud.delete_container(name)
        mock_swift.delete_container.assert_called_once_with(container=name)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_delete_container_error(self, mock_swift):
        """Non-404 swift error re-raised as OSCE"""
        mock_swift.delete_container.side_effect = swift_exc.ClientException(
            'ERROR')
        self.assertRaises(shade.OpenStackCloudException,
                          self.cloud.delete_container, '')

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_update_container(self, mock_swift):
        name = 'test_container'
        headers = {'x-container-read':
                   shade.openstackcloud.OBJECT_CONTAINER_ACLS['public']}
        self.cloud.update_container(name, headers)
        mock_swift.post_container.assert_called_once_with(
            container=name, headers=headers)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_update_container_error(self, mock_swift):
        """Swift error re-raised as OSCE"""
        mock_swift.post_container.side_effect = swift_exc.ClientException(
            'ERROR')
        self.assertRaises(shade.OpenStackCloudException,
                          self.cloud.update_container, '', '')

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_set_container_access_public(self, mock_swift):
        name = 'test_container'
        self.cloud.set_container_access(name, 'public')
        mock_swift.post_container.assert_called_once_with(
            container=name,
            headers={'x-container-read':
                     shade.openstackcloud.OBJECT_CONTAINER_ACLS['public']})

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_set_container_access_private(self, mock_swift):
        name = 'test_container'
        self.cloud.set_container_access(name, 'private')
        mock_swift.post_container.assert_called_once_with(
            container=name,
            headers={'x-container-read':
                     shade.openstackcloud.OBJECT_CONTAINER_ACLS['private']})

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_set_container_access_invalid(self, mock_swift):
        self.assertRaises(shade.OpenStackCloudException,
                          self.cloud.set_container_access, '', 'invalid')

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_container(self, mock_swift):
        fake_container = {
            'x-container-read':
            shade.openstackcloud.OBJECT_CONTAINER_ACLS['public']
        }
        mock_swift.head_container.return_value = fake_container
        access = self.cloud.get_container_access('foo')
        self.assertEqual('public', access)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_container_invalid(self, mock_swift):
        fake_container = {'x-container-read': 'invalid'}
        mock_swift.head_container.return_value = fake_container
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Could not determine container access for ACL: invalid"
        ):
            self.cloud.get_container_access('foo')

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_container_access_not_found(self, mock_swift):
        name = 'invalid_container'
        mock_swift.head_container.return_value = None
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Container not found: %s" % name
        ):
            self.cloud.get_container_access(name)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_list_containers(self, mock_swift):
        containers = [dict(id='1', name='containter1')]
        mock_swift.get_account.return_value = ('response_headers', containers)
        ret = self.cloud.list_containers()
        mock_swift.get_account.assert_called_once_with(full_listing=True)
        self.assertEqual(containers, ret)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_list_containers_not_full(self, mock_swift):
        containers = [dict(id='1', name='containter1')]
        mock_swift.get_account.return_value = ('response_headers', containers)
        ret = self.cloud.list_containers(full_listing=False)
        mock_swift.get_account.assert_called_once_with(full_listing=False)
        self.assertEqual(containers, ret)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_list_containers_exception(self, mock_swift):
        mock_swift.get_account.side_effect = swift_exc.ClientException("ERROR")
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.list_containers)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_list_objects(self, mock_swift):
        objects = [dict(id='1', name='object1')]
        mock_swift.get_container.return_value = ('response_headers', objects)
        ret = self.cloud.list_objects('container_name')
        mock_swift.get_container.assert_called_once_with(
            container='container_name', full_listing=True)
        self.assertEqual(objects, ret)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_list_objects_not_full(self, mock_swift):
        objects = [dict(id='1', name='object1')]
        mock_swift.get_container.return_value = ('response_headers', objects)
        ret = self.cloud.list_objects('container_name', full_listing=False)
        mock_swift.get_container.assert_called_once_with(
            container='container_name', full_listing=False)
        self.assertEqual(objects, ret)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_list_objects_exception(self, mock_swift):
        mock_swift.get_container.side_effect = swift_exc.ClientException(
            "ERROR")
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.list_objects, 'container_name')

    @mock.patch.object(shade.OpenStackCloud, 'get_object_metadata')
    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_delete_object(self, mock_swift, mock_get_meta):
        container_name = 'container_name'
        object_name = 'object_name'
        mock_get_meta.return_value = {'object': object_name}
        self.assertTrue(self.cloud.delete_object(container_name, object_name))
        mock_get_meta.assert_called_once_with(container_name, object_name)
        mock_swift.delete_object.assert_called_once_with(
            container=container_name, obj=object_name
        )

    @mock.patch.object(shade.OpenStackCloud, 'get_object_metadata')
    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_delete_object_not_found(self, mock_swift, mock_get_meta):
        container_name = 'container_name'
        object_name = 'object_name'
        mock_get_meta.return_value = None
        self.assertFalse(self.cloud.delete_object(container_name, object_name))
        mock_get_meta.assert_called_once_with(container_name, object_name)
        self.assertFalse(mock_swift.delete_object.called)

    @mock.patch.object(shade.OpenStackCloud, 'get_object_metadata')
    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_delete_object_exception(self, mock_swift, mock_get_meta):
        container_name = 'container_name'
        object_name = 'object_name'
        mock_get_meta.return_value = {'object': object_name}
        mock_swift.delete_object.side_effect = swift_exc.ClientException(
            "ERROR")
        self.assertRaises(shade.OpenStackCloudException,
                          self.cloud.delete_object,
                          container_name, object_name)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_object(self, mock_swift):
        fake_resp = ({'headers': 'yup'}, 'test body')
        mock_swift.get_object.return_value = fake_resp
        container_name = 'container_name'
        object_name = 'object_name'
        resp = self.cloud.get_object(container_name, object_name)
        self.assertEqual(fake_resp, resp)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_object_not_found(self, mock_swift):
        mock_swift.get_object.side_effect = swift_exc.ClientException(
            'ERROR', http_status=404)
        container_name = 'container_name'
        object_name = 'object_name'
        self.assertIsNone(self.cloud.get_object(container_name, object_name))
        mock_swift.get_object.assert_called_once_with(
            container=container_name, obj=object_name,
            query_string=None, resp_chunk_size=None)

    @mock.patch.object(shade.OpenStackCloud, 'swift_client')
    def test_get_object_exception(self, mock_swift):
        mock_swift.get_object.side_effect = swift_exc.ClientException("ERROR")
        container_name = 'container_name'
        object_name = 'object_name'
        self.assertRaises(shade.OpenStackCloudException,
                          self.cloud.get_object,
                          container_name, object_name)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_floating_ip_pool
----------------------------------

Test floating IP pool resource (managed by nova)
"""

from mock import patch
import os_client_config
from shade import OpenStackCloud
from shade import OpenStackCloudException
from shade.tests.unit import base
from shade.tests.fakes import FakeFloatingIPPool


class TestFloatingIPPool(base.TestCase):
    mock_pools = [
        {'id': 'pool1_id', 'name': 'pool1'},
        {'id': 'pool2_id', 'name': 'pool2'}]

    def setUp(self):
        super(TestFloatingIPPool, self).setUp()
        config = os_client_config.OpenStackConfig()
        self.client = OpenStackCloud(
            cloud_config=config.get_one_cloud(validate=False))

    @patch.object(OpenStackCloud, '_has_nova_extension')
    @patch.object(OpenStackCloud, 'nova_client')
    def test_list_floating_ip_pools(
            self, mock_nova_client, mock__has_nova_extension):
        mock_nova_client.floating_ip_pools.list.return_value = [
            FakeFloatingIPPool(**p) for p in self.mock_pools
        ]
        mock__has_nova_extension.return_value = True

        floating_ip_pools = self.client.list_floating_ip_pools()

        self.assertItemsEqual(floating_ip_pools, self.mock_pools)

    @patch.object(OpenStackCloud, '_has_nova_extension')
    @patch.object(OpenStackCloud, 'nova_client')
    def test_list_floating_ip_pools_exception(
            self, mock_nova_client, mock__has_nova_extension):
        mock_nova_client.floating_ip_pools.list.side_effect = \
            Exception('whatever')
        mock__has_nova_extension.return_value = True

        self.assertRaises(
            OpenStackCloudException, self.client.list_floating_ip_pools)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_domain
----------------------------------

Functional tests for `shade` keystone domain resource.
"""

import shade
from shade.tests import base


class TestDomain(base.TestCase):

    def setUp(self):
        super(TestDomain, self).setUp()
        self.cloud = shade.operator_cloud(cloud='devstack-admin')
        if self.cloud.cloud_config.get_api_version('identity') in ('2', '2.0'):
            self.skipTest('Identity service does not support domains')
        self.domain_prefix = self.getUniqueString('domain')
        self.addCleanup(self._cleanup_domains)

    def _cleanup_domains(self):
        exception_list = list()
        for domain in self.cloud.list_domains():
            if domain['name'].startswith(self.domain_prefix):
                try:
                    self.cloud.delete_domain(domain['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue

        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise shade.OpenStackCloudException('\n'.join(exception_list))

    def test_search_domains(self):
        domain_name = self.domain_prefix + '_search'

        # Shouldn't find any domain with this name yet
        results = self.cloud.search_domains(filters=dict(name=domain_name))
        self.assertEqual(0, len(results))

        # Now create a new domain
        domain = self.cloud.create_domain(domain_name)
        self.assertEqual(domain_name, domain['name'])

        # Now we should find only the new domain
        results = self.cloud.search_domains(filters=dict(name=domain_name))
        self.assertEqual(1, len(results))
        self.assertEqual(domain_name, results[0]['name'])

    def test_update_domain(self):
        domain = self.cloud.create_domain(self.domain_prefix, 'description')
        self.assertEqual(self.domain_prefix, domain['name'])
        self.assertEqual('description', domain['description'])
        self.assertTrue(domain['enabled'])
        updated = self.cloud.update_domain(domain['id'], name='updated name',
                                           description='updated description',
                                           enabled=False)
        self.assertEqual('updated name', updated['name'])
        self.assertEqual('updated description', updated['description'])
        self.assertFalse(updated['enabled'])



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_floating_ip_common
----------------------------------

Tests floating IP resource methods for Neutron and Nova-network.
"""

from mock import patch
import os_client_config
from shade import meta
from shade import OpenStackCloud
from shade.tests.fakes import FakeServer
from shade.tests.unit import base


class TestFloatingIP(base.TestCase):
    def setUp(self):
        super(TestFloatingIP, self).setUp()
        config = os_client_config.OpenStackConfig()
        self.client = OpenStackCloud(
            cloud_config=config.get_one_cloud(validate=False))

    @patch.object(OpenStackCloud, 'get_floating_ip')
    @patch.object(OpenStackCloud, '_attach_ip_to_server')
    @patch.object(OpenStackCloud, 'available_floating_ip')
    def test_add_auto_ip(
            self, mock_available_floating_ip, mock_attach_ip_to_server,
            mock_get_floating_ip):
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)
        floating_ip_dict = {
            "id": "this-is-a-floating-ip-id",
            "fixed_ip_address": None,
            "floating_ip_address": "203.0.113.29",
            "network": "this-is-a-net-or-pool-id",
            "attached": False,
            "status": "ACTIVE"
        }

        mock_available_floating_ip.return_value = floating_ip_dict

        self.client.add_auto_ip(server=server_dict)

        mock_attach_ip_to_server.assert_called_with(
            timeout=60, wait=False, server=server_dict,
            floating_ip=floating_ip_dict, skip_attach=False)

    @patch.object(OpenStackCloud, 'nova_client')
    @patch.object(OpenStackCloud, '_add_ip_from_pool')
    def test_add_ips_to_server_pool(
            self, mock_add_ip_from_pool, mock_nova_client):
        server = FakeServer(
            id='romeo', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)
        pool = 'nova'

        mock_nova_client.servers.get.return_value = server

        self.client.add_ips_to_server(server_dict, ip_pool=pool)

        mock_add_ip_from_pool.assert_called_with(
            server_dict, pool, reuse=True, wait=False, timeout=60,
            fixed_address=None)

    @patch.object(OpenStackCloud, 'nova_client')
    @patch.object(OpenStackCloud, 'add_ip_list')
    def test_add_ips_to_server_ip_list(
            self, mock_add_ip_list, mock_nova_client):
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)
        ips = ['203.0.113.29', '172.24.4.229']
        mock_nova_client.servers.get.return_value = server

        self.client.add_ips_to_server(server_dict, ips=ips)

        mock_add_ip_list.assert_called_with(
            server_dict, ips, wait=False, timeout=60, fixed_address=None)

    @patch.object(OpenStackCloud, 'nova_client')
    @patch.object(OpenStackCloud, '_add_auto_ip')
    def test_add_ips_to_server_auto_ip(
            self, mock_add_auto_ip, mock_nova_client):
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)

        mock_nova_client.servers.get.return_value = server

        self.client.add_ips_to_server(server_dict)

        mock_add_auto_ip.assert_called_with(
            server_dict, wait=False, timeout=60, reuse=True)



import os
import sys

sys.path.insert(0, os.path.abspath('../..'))

extensions = [
    'sphinx.ext.autodoc',
    'oslosphinx',
    'reno.sphinxext'
]

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'shade'
copyright = u'2014 Hewlett-Packard Development Company, L.P.'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index',
     '%s.tex' % project,
     u'%s Documentation' % project,
     u'Monty Taylor', 'manual'),
]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from mock import patch
import os_client_config as occ
from shade import OperatorCloud, operator_cloud
from shade.exc import OpenStackCloudException, OpenStackCloudTimeout
from shade.meta import obj_to_dict
from shade.tests import base, fakes
import testtools


class TestRoleAssignment(base.TestCase):

    def setUp(self):
        super(TestRoleAssignment, self).setUp()
        self.cloud = operator_cloud(validate=False)
        self.fake_role = obj_to_dict(fakes.FakeRole('12345', 'test'))
        self.fake_user = obj_to_dict(fakes.FakeUser('12345',
                                                    'test@nobody.org',
                                                    'test',
                                                    domain_id='test-domain'))
        self.fake_group = obj_to_dict(fakes.FakeGroup('12345',
                                                      'test',
                                                      'test group',
                                                      domain_id='test-domain'))
        self.fake_project = obj_to_dict(
            fakes.FakeProject('12345', domain_id='test-domain'))
        self.fake_domain = obj_to_dict(fakes.FakeDomain('test-domain',
                                                        'test',
                                                        'test domain',
                                                        enabled=True))
        self.user_project_assignment = obj_to_dict({
            'role': {
                'id': self.fake_role['id']
            },
            'scope': {
                'project': {
                    'id': self.fake_project['id']
                }
            },
            'user': {
                'id': self.fake_user['id']
            }
        })
        self.group_project_assignment = obj_to_dict({
            'role': {
                'id': self.fake_role['id']
            },
            'scope': {
                'project': {
                    'id': self.fake_project['id']
                }
            },
            'group': {
                'id': self.fake_group['id']
            }
        })
        self.user_domain_assignment = obj_to_dict({
            'role': {
                'id': self.fake_role['id']
            },
            'scope': {
                'domain': {
                    'id': self.fake_domain['id']
                }
            },
            'user': {
                'id': self.fake_user['id']
            }
        })
        self.group_domain_assignment = obj_to_dict({
            'role': {
                'id': self.fake_role['id']
            },
            'scope': {
                'domain': {
                    'id': self.fake_domain['id']
                }
            },
            'group': {
                'id': self.fake_group['id']
            }
        })

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_v2(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.roles.roles_for_user.return_value = []
        mock_keystone.roles.add_user_role.return_value = self.fake_role
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['name'],
                                              project=self.fake_project['id']))
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['id'],
                                              project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_project_v2(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.return_value = []
        mock_keystone.roles.add_user_role.return_value = self.fake_role
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['name'],
                                              project=self.fake_project['id']))
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['id'],
                                              project=self.fake_project['id']))
        self.assertTrue(self.cloud.grant_role(self.fake_role['id'],
                                              user=self.fake_user['name'],
                                              project=self.fake_project['id']))
        self.assertTrue(self.cloud.grant_role(self.fake_role['id'],
                                              user=self.fake_user['id'],
                                              project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_project_v2_exists(self,
                                               mock_keystone,
                                               mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.return_value = [self.fake_role]
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_project(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.role_assignments.list.return_value = []
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['name'],
                                              project=self.fake_project['id']))
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['id'],
                                              project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_project_exists(self,
                                            mock_keystone,
                                            mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.role_assignments.list.return_value = \
            [self.user_project_assignment]
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['id'],
            user=self.fake_user['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_group_project(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.role_assignments.list.return_value = []
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            project=self.fake_project['id']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_group_project_exists(self,
                                             mock_keystone,
                                             mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.role_assignments.list.return_value = \
            [self.group_project_assignment]
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_domain(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = []
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['id']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['id']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['name']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_domain_exists(self,
                                           mock_keystone,
                                           mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = \
            [self.user_domain_assignment]
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['id']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_group_domain(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = []
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['id']))
        self.assertTrue(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_group_domain_exists(self,
                                            mock_keystone,
                                            mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = \
            [self.group_domain_assignment]
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['id']))
        self.assertFalse(self.cloud.grant_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_v2(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.roles.roles_for_user.return_value = [self.fake_role]
        mock_keystone.roles.remove_user_role.return_value = self.fake_role
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_project_v2(self,
                                         mock_keystone,
                                         mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.return_value = []
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['id'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['id'],
            user=self.fake_user['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_project_v2_exists(self,
                                                mock_keystone,
                                                mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.return_value = [self.fake_role]
        mock_keystone.roles.remove_user_role.return_value = self.fake_role
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_project(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.role_assignments.list.return_value = []
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_project_exists(self,
                                             mock_keystone,
                                             mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.role_assignments.list.return_value = \
            [self.user_project_assignment]
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['id'],
            user=self.fake_user['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_group_project(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.role_assignments.list.return_value = []
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            project=self.fake_project['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_group_project_exists(self,
                                              mock_keystone,
                                              mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.role_assignments.list.return_value = \
            [self.group_project_assignment]
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            project=self.fake_project['id']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            project=self.fake_project['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_domain(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = []
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['name']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_domain_exists(self,
                                            mock_keystone,
                                            mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = \
            [self.user_domain_assignment]
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            domain=self.fake_domain['id']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['id'],
            domain=self.fake_domain['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_group_domain(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = []
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['name']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['id']))
        self.assertFalse(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_group_domain_exists(self,
                                             mock_keystone,
                                             mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.groups.list.return_value = [self.fake_group]
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = \
            [self.group_domain_assignment]
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['name']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['name'],
            domain=self.fake_domain['id']))
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            group=self.fake_group['id'],
            domain=self.fake_domain['id']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_no_role(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = []

        with testtools.ExpectedException(
            OpenStackCloudException,
            'Role {0} not found'.format(self.fake_role['name'])
        ):
            self.cloud.grant_role(self.fake_role['name'],
                                  group=self.fake_group['name'],
                                  domain=self.fake_domain['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_no_role(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = []
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Role {0} not found'.format(self.fake_role['name'])
        ):
            self.cloud.revoke_role(self.fake_role['name'],
                                   group=self.fake_group['name'],
                                   domain=self.fake_domain['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_no_user_or_group_specified(self,
                                              mock_keystone,
                                              mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Must specify either a user or a group'
        ):
            self.cloud.grant_role(self.fake_role['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_no_user_or_group_specified(self,
                                               mock_keystone,
                                               mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Must specify either a user or a group'
        ):
            self.cloud.revoke_role(self.fake_role['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_no_user_or_group(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = []
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Must specify either a user or a group'
        ):
            self.cloud.grant_role(self.fake_role['name'],
                                  user=self.fake_user['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_no_user_or_group(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = []
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Must specify either a user or a group'
        ):
            self.cloud.revoke_role(self.fake_role['name'],
                                   user=self.fake_user['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_both_user_and_group(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.groups.list.return_value = [self.fake_group]
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Specify either a group or a user, not both'
        ):
            self.cloud.grant_role(self.fake_role['name'],
                                  user=self.fake_user['name'],
                                  group=self.fake_group['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_both_user_and_group(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.groups.list.return_value = [self.fake_group]
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Specify either a group or a user, not both'
        ):
            self.cloud.revoke_role(self.fake_role['name'],
                                   user=self.fake_user['name'],
                                   group=self.fake_group['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_both_project_and_domain(self,
                                           mock_keystone,
                                           mock_api_version):
        mock_api_version.return_value = '3'
        fake_user2 = fakes.FakeUser('12345',
                                    'test@nobody.org',
                                    'test',
                                    domain_id='default')
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user, fake_user2]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.domains.get.return_value = self.fake_domain
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['name'],
                                              project=self.fake_project['id'],
                                              domain=self.fake_domain['name']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_both_project_and_domain(self,
                                            mock_keystone,
                                            mock_api_version):
        mock_api_version.return_value = '3'
        fake_user2 = fakes.FakeUser('12345',
                                    'test@nobody.org',
                                    'test',
                                    domain_id='default')
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user, fake_user2]
        mock_keystone.projects.list.return_value = [self.fake_project]
        mock_keystone.domains.get.return_value = self.fake_domain
        mock_keystone.role_assignments.list.return_value = \
            [self.user_project_assignment]
        self.assertTrue(self.cloud.revoke_role(
            self.fake_role['name'],
            user=self.fake_user['name'],
            project=self.fake_project['id'],
            domain=self.fake_domain['name']))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_no_project_or_domain(self, mock_keystone, mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.projects.list.return_value = []
        mock_keystone.domains.get.return_value = None
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Must specify either a domain or project'
        ):
            self.cloud.grant_role(self.fake_role['name'],
                                  user=self.fake_user['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_no_project_or_domain(self,
                                         mock_keystone,
                                         mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.projects.list.return_value = []
        mock_keystone.domains.get.return_value = None
        mock_keystone.role_assignments.list.return_value = \
            [self.user_project_assignment]
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Must specify either a domain or project'
        ):
            self.cloud.revoke_role(self.fake_role['name'],
                                   user=self.fake_user['name'])

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_bad_domain_exception(self,
                                        mock_keystone,
                                        mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.side_effect = Exception('test')
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Failed to get domain baddomain \(Inner Exception: test\)'
        ):
            self.cloud.grant_role(self.fake_role['name'],
                                  user=self.fake_user['name'],
                                  domain='baddomain')

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_bad_domain_exception(self,
                                         mock_keystone,
                                         mock_api_version):
        mock_api_version.return_value = '3'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.domains.get.side_effect = Exception('test')
        with testtools.ExpectedException(
            OpenStackCloudException,
            'Failed to get domain baddomain \(Inner Exception: test\)'
        ):
            self.cloud.revoke_role(self.fake_role['name'],
                                   user=self.fake_user['name'],
                                   domain='baddomain')

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_project_v2_wait(self,
                                             mock_keystone,
                                             mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.side_effect = [
            [], [], [self.fake_role]]
        mock_keystone.roles.add_user_role.return_value = self.fake_role
        self.assertTrue(self.cloud.grant_role(self.fake_role['name'],
                                              user=self.fake_user['name'],
                                              project=self.fake_project['id'],
                                              wait=True))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_grant_role_user_project_v2_wait_exception(self,
                                                       mock_keystone,
                                                       mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.side_effect = [
            [], [], [self.fake_role]]
        mock_keystone.roles.add_user_role.return_value = self.fake_role

        with testtools.ExpectedException(
            OpenStackCloudTimeout,
            'Timeout waiting for role to be granted'
        ):
            self.assertTrue(self.cloud.grant_role(
                self.fake_role['name'], user=self.fake_user['name'],
                project=self.fake_project['id'], wait=True, timeout=1))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_project_v2_wait(self,
                                              mock_keystone,
                                              mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.side_effect = [
            [self.fake_role], [self.fake_role],
            []]
        mock_keystone.roles.remove_user_role.return_value = self.fake_role
        self.assertTrue(self.cloud.revoke_role(self.fake_role['name'],
                                               user=self.fake_user['name'],
                                               project=self.fake_project['id'],
                                               wait=True))

    @patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
    @patch.object(OperatorCloud, 'keystone_client')
    def test_revoke_role_user_project_v2_wait_exception(self,
                                                        mock_keystone,
                                                        mock_api_version):
        mock_api_version.return_value = '2.0'
        mock_keystone.roles.list.return_value = [self.fake_role]
        mock_keystone.tenants.list.return_value = [self.fake_project]
        mock_keystone.users.list.return_value = [self.fake_user]
        mock_keystone.roles.roles_for_user.side_effect = [
            [self.fake_role], [self.fake_role],
            []]
        mock_keystone.roles.remove_user_role.return_value = self.fake_role
        with testtools.ExpectedException(
            OpenStackCloudTimeout,
            'Timeout waiting for role to be revoked'
        ):
            self.assertTrue(self.cloud.revoke_role(
                self.fake_role['name'], user=self.fake_user['name'],
                project=self.fake_project['id'], wait=True, timeout=1))



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_endpoint
----------------------------------

Functional tests for `shade` endpoint resource.
"""

import string
import random

from shade import operator_cloud
from shade.exc import OpenStackCloudException
from shade.tests import base


class TestEndpoints(base.TestCase):

    endpoint_attributes = ['id', 'region', 'publicurl', 'internalurl',
                           'service_id', 'adminurl']

    def setUp(self):
        super(TestEndpoints, self).setUp()
        self.operator_cloud = operator_cloud(cloud='devstack-admin')

        # Generate a random name for services and regions in this test
        self.new_item_name = 'test_' + ''.join(
            random.choice(string.ascii_lowercase) for _ in range(5))

        self.addCleanup(self._cleanup_services)
        self.addCleanup(self._cleanup_endpoints)

    def _cleanup_endpoints(self):
        exception_list = list()
        for e in self.operator_cloud.list_endpoints():
            if e.get('region') is not None and \
                    e['region'].startswith(self.new_item_name):
                try:
                    self.operator_cloud.delete_endpoint(id=e['id'])
                except Exception as e:
                    # We were unable to delete a service, let's try with next
                    exception_list.append(str(e))
                    continue
        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise OpenStackCloudException('\n'.join(exception_list))

    def _cleanup_services(self):
        exception_list = list()
        for s in self.operator_cloud.list_services():
            if s['name'] is not None and \
                    s['name'].startswith(self.new_item_name):
                try:
                    self.operator_cloud.delete_service(name_or_id=s['id'])
                except Exception as e:
                    # We were unable to delete a service, let's try with next
                    exception_list.append(str(e))
                    continue
        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise OpenStackCloudException('\n'.join(exception_list))

    def test_create_endpoint(self):
        service_name = self.new_item_name + '_create'

        service = self.operator_cloud.create_service(
            name=service_name, type='test_type',
            description='this is a test description')

        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            internal_url='http://internal.test/',
            admin_url='http://admin.url/',
            region=service_name)

        self.assertNotEqual([], endpoints)
        self.assertIsNotNone(endpoints[0].get('id'))

        # Test None parameters
        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            region=service_name)

        self.assertNotEqual([], endpoints)
        self.assertIsNotNone(endpoints[0].get('id'))

    def test_list_endpoints(self):
        service_name = self.new_item_name + '_list'

        service = self.operator_cloud.create_service(
            name=service_name, type='test_type',
            description='this is a test description')

        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            internal_url='http://internal.test/',
            region=service_name)

        observed_endpoints = self.operator_cloud.list_endpoints()
        found = False
        for e in observed_endpoints:
            # Test all attributes are returned
            for endpoint in endpoints:
                if e['id'] == endpoint['id']:
                    found = True
                    self.assertEqual(service['id'], e['service_id'])
                    if 'interface' in e:
                        if 'interface' == 'internal':
                            self.assertEqual('http://internal.test/', e['url'])
                        elif 'interface' == 'public':
                            self.assertEqual('http://public.test/', e['url'])
                    else:
                        self.assertEqual('http://public.test/',
                                         e['publicurl'])
                        self.assertEqual('http://internal.test/',
                                         e['internalurl'])
                    self.assertEqual(service_name, e['region'])

        self.assertTrue(found, msg='new endpoint not found in endpoints list!')

    def test_delete_endpoint(self):
        service_name = self.new_item_name + '_delete'

        service = self.operator_cloud.create_service(
            name=service_name, type='test_type',
            description='this is a test description')

        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            internal_url='http://internal.test/',
            region=service_name)

        self.assertNotEqual([], endpoints)
        for endpoint in endpoints:
            self.operator_cloud.delete_endpoint(endpoint['id'])

        observed_endpoints = self.operator_cloud.list_endpoints()
        found = False
        for e in observed_endpoints:
            for endpoint in endpoints:
                if e['id'] == endpoint['id']:
                    found = True
                    break
        self.failUnlessEqual(
            False, found, message='new endpoint was not deleted!')



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import plugin as ksa_plugin

import mock
import testtools

from os_client_config import cloud_config
import shade
from shade import exc
from shade import meta
from shade.tests import fakes
from shade.tests.unit import base


class TestShadeOperator(base.TestCase):

    def setUp(self):
        super(TestShadeOperator, self).setUp()
        self.cloud = shade.operator_cloud(validate=False)

    def test_operator_cloud(self):
        self.assertIsInstance(self.cloud, shade.OperatorCloud)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_get_machine(self, mock_client):
        node = fakes.FakeMachine(id='00000000-0000-0000-0000-000000000000',
                                 name='bigOlFaker')
        mock_client.node.get.return_value = node
        machine = self.cloud.get_machine('bigOlFaker')
        mock_client.node.get.assert_called_with(node_id='bigOlFaker')
        self.assertEqual(meta.obj_to_dict(node), machine)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_get_machine_by_mac(self, mock_client):
        class port_value:
            node_uuid = '00000000-0000-0000-0000-000000000000'
            address = '00:00:00:00:00:00'

        class node_value:
            uuid = '00000000-0000-0000-0000-000000000000'

        expected_value = dict(
            uuid='00000000-0000-0000-0000-000000000000')

        mock_client.port.get_by_address.return_value = port_value
        mock_client.node.get.return_value = node_value
        machine = self.cloud.get_machine_by_mac('00:00:00:00:00:00')
        mock_client.port.get_by_address.assert_called_with(
            address='00:00:00:00:00:00')
        mock_client.node.get.assert_called_with(
            node_id='00000000-0000-0000-0000-000000000000')
        self.assertEqual(machine, expected_value)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_list_machines(self, mock_client):
        m1 = fakes.FakeMachine(1, 'fake_machine1')
        mock_client.node.list.return_value = [m1]
        machines = self.cloud.list_machines()
        self.assertTrue(mock_client.node.list.called)
        self.assertEqual(meta.obj_to_dict(m1), machines[0])

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_validate_node(self, mock_client):
        node_uuid = '123'
        self.cloud.validate_node(node_uuid)
        mock_client.node.validate.assert_called_once_with(
            node_uuid=node_uuid
        )

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_list_nics(self, mock_client):
        port1 = fakes.FakeMachinePort(1, "aa:bb:cc:dd", "node1")
        port2 = fakes.FakeMachinePort(2, "dd:cc:bb:aa", "node2")
        port_list = [port1, port2]
        port_dict_list = meta.obj_list_to_dict(port_list)

        mock_client.port.list.return_value = port_list
        nics = self.cloud.list_nics()

        self.assertTrue(mock_client.port.list.called)
        self.assertEqual(port_dict_list, nics)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_list_nics_failure(self, mock_client):
        mock_client.port.list.side_effect = Exception()
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.list_nics)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_list_nics_for_machine(self, mock_client):
        mock_client.node.list_ports.return_value = []
        self.cloud.list_nics_for_machine("123")
        mock_client.node.list_ports.assert_called_with(node_id="123")

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_list_nics_for_machine_failure(self, mock_client):
        mock_client.node.list_ports.side_effect = Exception()
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.list_nics_for_machine, None)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_patch_machine(self, mock_client):
        node_id = 'node01'
        patch = []
        patch.append({'op': 'remove', 'path': '/instance_info'})
        self.cloud.patch_machine(node_id, patch)
        self.assertTrue(mock_client.node.update.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_no_action(self, mock_patch, mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            name = 'node01'

        expected_machine = dict(
            uuid='00000000-0000-0000-0000-000000000000',
            name='node01'
        )
        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine('node01')
        self.assertIsNone(update_dict['changes'])
        self.assertFalse(mock_patch.called)
        self.assertDictEqual(expected_machine, update_dict['node'])

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_no_action_name(self, mock_patch,
                                                 mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            name = 'node01'

        expected_machine = dict(
            uuid='00000000-0000-0000-0000-000000000000',
            name='node01'
        )
        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine('node01', name='node01')
        self.assertIsNone(update_dict['changes'])
        self.assertFalse(mock_patch.called)
        self.assertDictEqual(expected_machine, update_dict['node'])

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_action_name(self, mock_patch,
                                              mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            name = 'evil'

        expected_patch = [dict(op='replace', path='/name', value='good')]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine('evil', name='good')
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/name', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_name(self, mock_patch,
                                              mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            name = 'evil'

        expected_patch = [dict(op='replace', path='/name', value='good')]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine('evil', name='good')
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/name', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_chassis_uuid(self, mock_patch,
                                                      mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            chassis_uuid = None

        expected_patch = [
            dict(
                op='replace',
                path='/chassis_uuid',
                value='00000000-0000-0000-0000-000000000001'
            )]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine(
            '00000000-0000-0000-0000-000000000000',
            chassis_uuid='00000000-0000-0000-0000-000000000001')
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/chassis_uuid', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_driver(self, mock_patch,
                                                mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            driver = None

        expected_patch = [
            dict(
                op='replace',
                path='/driver',
                value='fake'
            )]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine(
            '00000000-0000-0000-0000-000000000000',
            driver='fake'
        )
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/driver', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_driver_info(self, mock_patch,
                                                     mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            driver_info = None

        expected_patch = [
            dict(
                op='replace',
                path='/driver_info',
                value=dict(var='fake')
            )]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine(
            '00000000-0000-0000-0000-000000000000',
            driver_info=dict(var="fake")
        )
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/driver_info', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_instance_info(self, mock_patch,
                                                       mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            instance_info = None

        expected_patch = [
            dict(
                op='replace',
                path='/instance_info',
                value=dict(var='fake')
            )]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine(
            '00000000-0000-0000-0000-000000000000',
            instance_info=dict(var="fake")
        )
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/instance_info', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_instance_uuid(self, mock_patch,
                                                       mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            instance_uuid = None

        expected_patch = [
            dict(
                op='replace',
                path='/instance_uuid',
                value='00000000-0000-0000-0000-000000000002'
            )]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine(
            '00000000-0000-0000-0000-000000000000',
            instance_uuid='00000000-0000-0000-0000-000000000002'
        )
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/instance_uuid', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'patch_machine')
    def test_update_machine_patch_update_properties(self, mock_patch,
                                                    mock_client):
        class client_return_value:
            uuid = '00000000-0000-0000-0000-000000000000'
            properties = None

        expected_patch = [
            dict(
                op='replace',
                path='/properties',
                value=dict(var='fake')
            )]

        mock_client.node.get.return_value = client_return_value

        update_dict = self.cloud.update_machine(
            '00000000-0000-0000-0000-000000000000',
            properties=dict(var="fake")
        )
        self.assertIsNotNone(update_dict['changes'])
        self.assertEqual('/properties', update_dict['changes'][0])
        self.assertTrue(mock_patch.called)
        mock_patch.assert_called_with(
            '00000000-0000-0000-0000-000000000000',
            expected_patch)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_fail_active(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class active_machine:
            uuid = machine_uuid
            provision_state = "active"

        mock_client.node.get.return_value = active_machine
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.inspect_machine,
            machine_uuid,
            wait=True,
            timeout=1)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_failed(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class inspect_failed_machine:
            uuid = machine_uuid
            provision_state = "inspect failed"
            last_error = "kaboom"

        mock_client.node.get.return_value = inspect_failed_machine
        self.cloud.inspect_machine(machine_uuid)
        self.assertTrue(mock_client.node.set_provision_state.called)
        self.assertEqual(
            mock_client.node.set_provision_state.call_count, 1)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_managable(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class manageable_machine:
            uuid = machine_uuid
            provision_state = "manageable"

        mock_client.node.get.return_value = manageable_machine
        self.cloud.inspect_machine(machine_uuid)
        self.assertEqual(
            mock_client.node.set_provision_state.call_count, 1)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_available(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class available_machine:
            uuid = machine_uuid
            provision_state = "available"

        class manageable_machine:
            uuid = machine_uuid
            provision_state = "manageable"

        class inspecting_machine:
            uuid = machine_uuid
            provision_state = "inspecting"

        mock_client.node.get.side_effect = iter([
            available_machine,
            available_machine,
            manageable_machine,
            manageable_machine,
            inspecting_machine])
        self.cloud.inspect_machine(machine_uuid)
        self.assertTrue(mock_client.node.set_provision_state.called)
        self.assertEqual(
            mock_client.node.set_provision_state.call_count, 3)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_available_wait(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class available_machine:
            uuid = machine_uuid
            provision_state = "available"

        class manageable_machine:
            uuid = machine_uuid
            provision_state = "manageable"

        class inspecting_machine:
            uuid = machine_uuid
            provision_state = "inspecting"

        mock_client.node.get.side_effect = iter([
            available_machine,
            available_machine,
            manageable_machine,
            inspecting_machine,
            manageable_machine,
            available_machine,
            available_machine])
        expected_return_value = dict(
            uuid=machine_uuid,
            provision_state="available"
        )

        return_value = self.cloud.inspect_machine(
            machine_uuid, wait=True, timeout=1)
        self.assertTrue(mock_client.node.set_provision_state.called)
        self.assertEqual(
            mock_client.node.set_provision_state.call_count, 3)
        self.assertDictEqual(expected_return_value, return_value)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_wait(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class manageable_machine:
            uuid = machine_uuid
            provision_state = "manageable"

        class inspecting_machine:
            uuid = machine_uuid
            provision_state = "inspecting"

        expected_return_value = dict(
            uuid=machine_uuid,
            provision_state="manageable"
        )
        mock_client.node.get.side_effect = iter([
            manageable_machine,
            inspecting_machine,
            inspecting_machine,
            manageable_machine,
            manageable_machine])

        return_value = self.cloud.inspect_machine(
            machine_uuid, wait=True, timeout=1)
        self.assertDictEqual(expected_return_value, return_value)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_inspect_machine_inspect_failed(self, mock_client):

        machine_uuid = '00000000-0000-0000-0000-000000000000'

        class manageable_machine:
            uuid = machine_uuid
            provision_state = "manageable"
            last_error = None

        class inspecting_machine:
            uuid = machine_uuid
            provision_state = "inspecting"
            last_error = None

        class inspect_failed_machine:
            uuid = machine_uuid
            provision_state = "inspect failed"
            last_error = "kaboom"

        mock_client.node.get.side_effect = iter([
            manageable_machine,
            inspecting_machine,
            inspect_failed_machine])
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.inspect_machine,
            machine_uuid,
            wait=True,
            timeout=1)
        self.assertEqual(
            mock_client.node.set_provision_state.call_count, 1)
        self.assertEqual(mock_client.node.get.call_count, 3)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_register_machine(self, mock_client):
        class fake_node:
            uuid = "00000000-0000-0000-0000-000000000000"
            provision_state = "available"
            reservation = None
            last_error = None

        expected_return_value = dict(
            uuid="00000000-0000-0000-0000-000000000000",
            provision_state="available",
            reservation=None,
            last_error=None
        )
        mock_client.node.create.return_value = fake_node
        mock_client.node.get.return_value = fake_node
        nics = [{'mac': '00:00:00:00:00:00'}]
        return_value = self.cloud.register_machine(nics)
        self.assertDictEqual(expected_return_value, return_value)
        self.assertTrue(mock_client.node.create.called)
        self.assertTrue(mock_client.port.create.called)
        self.assertFalse(mock_client.node.get.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'node_set_provision_state')
    def test_register_machine_enroll(
            self,
            mock_set_state,
            mock_client):
        machine_uuid = "00000000-0000-0000-0000-000000000000"

        class fake_node_init_state:
            uuid = machine_uuid
            provision_state = "enroll"
            reservation = None
            last_error = None

        class fake_node_post_manage:
            uuid = machine_uuid
            provision_state = "enroll"
            reservation = "do you have a flag?"
            last_error = None

        class fake_node_post_manage_done:
            uuid = machine_uuid
            provision_state = "manage"
            reservation = None
            last_error = None

        class fake_node_post_provide:
            uuid = machine_uuid
            provision_state = "available"
            reservation = None
            last_error = None

        class fake_node_post_enroll_failure:
            uuid = machine_uuid
            provision_state = "enroll"
            reservation = None
            last_error = "insufficent lolcats"

        expected_return_value = dict(
            uuid=machine_uuid,
            provision_state="available",
            reservation=None,
            last_error=None
        )

        mock_client.node.get.side_effect = iter([
            fake_node_init_state,
            fake_node_post_manage,
            fake_node_post_manage_done,
            fake_node_post_provide])
        mock_client.node.create.return_value = fake_node_init_state
        nics = [{'mac': '00:00:00:00:00:00'}]
        return_value = self.cloud.register_machine(nics)
        self.assertDictEqual(expected_return_value, return_value)
        self.assertTrue(mock_client.node.create.called)
        self.assertTrue(mock_client.port.create.called)
        self.assertTrue(mock_client.node.get.called)
        mock_client.reset_mock()
        mock_client.node.get.side_effect = iter([
            fake_node_init_state,
            fake_node_post_manage,
            fake_node_post_manage_done,
            fake_node_post_provide])
        return_value = self.cloud.register_machine(nics, wait=True)
        self.assertDictEqual(expected_return_value, return_value)
        self.assertTrue(mock_client.node.create.called)
        self.assertTrue(mock_client.port.create.called)
        self.assertTrue(mock_client.node.get.called)
        mock_client.reset_mock()
        mock_client.node.get.side_effect = iter([
            fake_node_init_state,
            fake_node_post_manage,
            fake_node_post_enroll_failure])
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.register_machine,
            nics)
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.register_machine,
            nics,
            wait=True)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade.OperatorCloud, 'node_set_provision_state')
    def test_register_machine_enroll_timeout(
            self,
            mock_set_state,
            mock_client):
        machine_uuid = "00000000-0000-0000-0000-000000000000"

        class fake_node_init_state:
            uuid = machine_uuid
            provision_state = "enroll"
            reservation = "do you have a flag?"
            last_error = None

        mock_client.node.get.return_value = fake_node_init_state
        mock_client.node.create.return_value = fake_node_init_state
        nics = [{'mac': '00:00:00:00:00:00'}]
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.register_machine,
            nics,
            lock_timeout=0.001)
        self.assertTrue(mock_client.node.create.called)
        self.assertTrue(mock_client.port.create.called)
        self.assertTrue(mock_client.node.get.called)
        mock_client.node.get.reset_mock()
        mock_client.node.create.reset_mock()
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.register_machine,
            nics,
            wait=True,
            timeout=0.001)
        self.assertTrue(mock_client.node.create.called)
        self.assertTrue(mock_client.port.create.called)
        self.assertTrue(mock_client.node.get.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_register_machine_port_create_failed(self, mock_client):
        class fake_node:
            uuid = "00000000-0000-0000-0000-000000000000"
            provision_state = "available"
            resevation = None
            last_error = None

        nics = [{'mac': '00:00:00:00:00:00'}]
        mock_client.node.create.return_value = fake_node
        mock_client.port.create.side_effect = (
            exc.OpenStackCloudException("Error"))
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.register_machine,
                          nics)
        self.assertTrue(mock_client.node.create.called)
        self.assertTrue(mock_client.port.create.called)
        self.assertTrue(mock_client.node.delete.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_unregister_machine(self, mock_client):
        class fake_node:
            provision_state = 'available'

        class fake_port:
            uuid = '00000000-0000-0000-0000-000000000001'

        mock_client.port.get_by_address.return_value = fake_port
        mock_client.node.get.return_value = fake_node
        nics = [{'mac': '00:00:00:00:00:00'}]
        uuid = "00000000-0000-0000-0000-000000000000"
        self.cloud.unregister_machine(nics, uuid)
        self.assertTrue(mock_client.node.delete.called)
        self.assertTrue(mock_client.port.get_by_address.called)
        self.assertTrue(mock_client.port.delete.called)
        self.assertTrue(mock_client.port.get_by_address.called)
        mock_client.port.get_by_address.assert_called_with(
            address='00:00:00:00:00:00')
        mock_client.port.delete.assert_called_with(
            port_id='00000000-0000-0000-0000-000000000001')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_unregister_machine_unavailable(self, mock_client):
        invalid_states = ['active', 'cleaning', 'clean wait', 'clean failed']
        nics = [{'mac': '00:00:00:00:00:00'}]
        uuid = "00000000-0000-0000-0000-000000000000"
        for state in invalid_states:
            class fake_node:
                provision_state = state

            mock_client.node.get.return_value = fake_node
            self.assertRaises(
                exc.OpenStackCloudException,
                self.cloud.unregister_machine,
                nics,
                uuid)
            self.assertFalse(mock_client.node.delete.called)
            self.assertFalse(mock_client.port.delete.called)
            self.assertFalse(mock_client.port.get_by_address.called)
            self.assertTrue(mock_client.node.get.called)
            mock_client.node.reset_mock()
            mock_client.node.reset_mock()

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_unregister_machine_timeout(self, mock_client):
        class fake_node:
            provision_state = 'available'

        mock_client.node.get.return_value = fake_node
        nics = [{'mac': '00:00:00:00:00:00'}]
        uuid = "00000000-0000-0000-0000-000000000000"
        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud.unregister_machine,
            nics,
            uuid,
            wait=True,
            timeout=0.001)
        self.assertTrue(mock_client.node.delete.called)
        self.assertTrue(mock_client.port.delete.called)
        self.assertTrue(mock_client.port.get_by_address.called)
        self.assertTrue(mock_client.node.get.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_machine_maintenace_state(self, mock_client):
        mock_client.node.set_maintenance.return_value = None
        node_id = 'node01'
        reason = 'no reason'
        self.cloud.set_machine_maintenance_state(node_id, True, reason=reason)
        mock_client.node.set_maintenance.assert_called_with(
            node_id='node01',
            state='true',
            maint_reason='no reason')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_machine_maintenace_state_false(self, mock_client):
        mock_client.node.set_maintenance.return_value = None
        node_id = 'node01'
        self.cloud.set_machine_maintenance_state(node_id, False)
        mock_client.node.set_maintenance.assert_called_with(
            node_id='node01',
            state='false')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_remove_machine_from_maintenance(self, mock_client):
        mock_client.node.set_maintenance.return_value = None
        node_id = 'node01'
        self.cloud.remove_machine_from_maintenance(node_id)
        mock_client.node.set_maintenance.assert_called_with(
            node_id='node01',
            state='false')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_machine_power_on(self, mock_client):
        mock_client.node.set_power_state.return_value = None
        node_id = 'node01'
        return_value = self.cloud.set_machine_power_on(node_id)
        self.assertEqual(None, return_value)
        mock_client.node.set_power_state.assert_called_with(
            node_id='node01',
            state='on')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_machine_power_off(self, mock_client):
        mock_client.node.set_power_state.return_value = None
        node_id = 'node01'
        return_value = self.cloud.set_machine_power_off(node_id)
        self.assertEqual(None, return_value)
        mock_client.node.set_power_state.assert_called_with(
            node_id='node01',
            state='off')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_machine_power_reboot(self, mock_client):
        mock_client.node.set_power_state.return_value = None
        node_id = 'node01'
        return_value = self.cloud.set_machine_power_reboot(node_id)
        self.assertEqual(None, return_value)
        mock_client.node.set_power_state.assert_called_with(
            node_id='node01',
            state='reboot')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_machine_power_reboot_failure(self, mock_client):
        mock_client.node.set_power_state.return_value = 'failure'
        self.assertRaises(shade.OpenStackCloudException,
                          self.cloud.set_machine_power_reboot,
                          'node01')
        mock_client.node.set_power_state.assert_called_with(
            node_id='node01',
            state='reboot')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_node_set_provision_state(self, mock_client):

        class active_node_state:
            provision_state = "active"

        active_return_value = dict(
            provision_state="active")

        mock_client.node.set_provision_state.return_value = None
        mock_client.node.get.return_value = active_node_state
        node_id = 'node01'
        return_value = self.cloud.node_set_provision_state(
            node_id,
            'active',
            configdrive='http://127.0.0.1/file.iso')
        self.assertEqual(active_return_value, return_value)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node01',
            state='active',
            configdrive='http://127.0.0.1/file.iso')
        self.assertTrue(mock_client.node.get.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_node_set_provision_state_wait_timeout(self, mock_client):
        class deploying_node_state:
            provision_state = "deploying"

        class active_node_state:
            provision_state = "active"

        class managable_node_state:
            provision_state = "managable"

        class available_node_state:
            provision_state = "available"

        active_return_value = dict(
            provision_state="active")
        mock_client.node.get.return_value = active_node_state
        mock_client.node.set_provision_state.return_value = None
        node_id = 'node01'
        return_value = self.cloud.node_set_provision_state(
            node_id,
            'active',
            configdrive='http://127.0.0.1/file.iso',
            wait=True)

        self.assertEqual(active_return_value, return_value)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node01',
            state='active',
            configdrive='http://127.0.0.1/file.iso')
        self.assertTrue(mock_client.node.get.called)
        mock_client.mock_reset()
        mock_client.node.get.return_value = deploying_node_state
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.node_set_provision_state,
            node_id,
            'active',
            configdrive='http://127.0.0.1/file.iso',
            wait=True,
            timeout=0.001)
        self.assertTrue(mock_client.node.get.called)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node01',
            state='active',
            configdrive='http://127.0.0.1/file.iso')

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_node_set_provision_state_wait_provide(self, mock_client):

        class managable_node_state:
            provision_state = "managable"

        class available_node_state:
            provision_state = "available"

        node_provide_return_value = dict(
            provision_state="available")

        mock_client.node.get.side_effect = iter([
            managable_node_state,
            available_node_state])
        return_value = self.cloud.node_set_provision_state(
            'test_node',
            'provide',
            wait=True)
        self.assertEqual(mock_client.node.get.call_count, 2)
        self.assertDictEqual(node_provide_return_value, return_value)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade._utils, '_iterate_timeout')
    def test_activate_node(self, mock_timeout, mock_client):
        mock_client.node.set_provision_state.return_value = None
        node_id = 'node02'
        return_value = self.cloud.activate_node(
            node_id,
            configdrive='http://127.0.0.1/file.iso')
        self.assertEqual(None, return_value)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node02',
            state='active',
            configdrive='http://127.0.0.1/file.iso')
        self.assertFalse(mock_timeout.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_activate_node_timeout(self, mock_client):

        class active_node_state:
            provision_state = 'active'

        class available_node_state:
            provision_state = 'available'

        mock_client.node.get.side_effect = iter([
            available_node_state,
            active_node_state])

        mock_client.node.set_provision_state.return_value = None
        node_id = 'node04'
        return_value = self.cloud.activate_node(
            node_id,
            configdrive='http://127.0.0.1/file.iso',
            wait=True,
            timeout=2)
        self.assertEqual(None, return_value)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node04',
            state='active',
            configdrive='http://127.0.0.1/file.iso')
        self.assertEqual(mock_client.node.get.call_count, 2)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    @mock.patch.object(shade._utils, '_iterate_timeout')
    def test_deactivate_node(self, mock_timeout, mock_client):
        mock_client.node.set_provision_state.return_value = None
        node_id = 'node03'
        return_value = self.cloud.deactivate_node(
            node_id, wait=False)
        self.assertEqual(None, return_value)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node03',
            state='deleted',
            configdrive=None)
        self.assertFalse(mock_timeout.called)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_deactivate_node_timeout(self, mock_client):

        class active_node_state:
            provision_state = 'active'

        class deactivated_node_state:
            provision_state = 'available'

        mock_client.node.get.side_effect = iter([
            active_node_state,
            deactivated_node_state])

        mock_client.node.set_provision_state.return_value = None
        node_id = 'node03'
        return_value = self.cloud.deactivate_node(
            node_id, wait=True, timeout=2)
        self.assertEqual(None, return_value)
        mock_client.node.set_provision_state.assert_called_with(
            node_uuid='node03',
            state='deleted',
            configdrive=None)
        self.assertEqual(mock_client.node.get.call_count, 2)

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_set_node_instance_info(self, mock_client):
        uuid = 'aaa'
        patch = [{'op': 'add', 'foo': 'bar'}]
        self.cloud.set_node_instance_info(uuid, patch)
        mock_client.node.update.assert_called_with(
            node_id=uuid, patch=patch
        )

    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
    def test_purge_node_instance_info(self, mock_client):
        uuid = 'aaa'
        expected_patch = [{'op': 'remove', 'path': '/instance_info'}]
        self.cloud.purge_node_instance_info(uuid)
        mock_client.node.update.assert_called_with(
            node_id=uuid, patch=expected_patch
        )

    @mock.patch.object(shade.OpenStackCloud, 'glance_client')
    def test_get_image_name(self, glance_mock):

        class Image(object):
            id = '22'
            name = '22 name'
            status = 'success'
        fake_image = Image()
        glance_mock.images.list.return_value = [fake_image]
        self.assertEqual('22 name', self.cloud.get_image_name('22'))
        self.assertEqual('22 name', self.cloud.get_image_name('22 name'))

    @mock.patch.object(shade.OpenStackCloud, 'glance_client')
    def test_get_image_id(self, glance_mock):

        class Image(object):
            id = '22'
            name = '22 name'
            status = 'success'
        fake_image = Image()
        glance_mock.images.list.return_value = [fake_image]
        self.assertEqual('22', self.cloud.get_image_id('22'))
        self.assertEqual('22', self.cloud.get_image_id('22 name'))

    @mock.patch.object(cloud_config.CloudConfig, 'get_endpoint')
    def test_get_session_endpoint_provided(self, fake_get_endpoint):
        fake_get_endpoint.return_value = 'http://fake.url'
        self.assertEqual(
            'http://fake.url', self.cloud.get_session_endpoint('image'))

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_get_session_endpoint_session(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = 'http://fake.url'
        get_session_mock.return_value = session_mock
        self.assertEqual(
            'http://fake.url', self.cloud.get_session_endpoint('image'))

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_get_session_endpoint_exception(self, get_session_mock):
        class FakeException(Exception):
            pass

        def side_effect(*args, **kwargs):
            raise FakeException("No service")
        session_mock = mock.Mock()
        session_mock.get_endpoint.side_effect = side_effect
        get_session_mock.return_value = session_mock
        self.cloud.name = 'testcloud'
        self.cloud.region_name = 'testregion'
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Error getting image endpoint on testcloud:testregion:"
                " No service"):
            self.cloud.get_session_endpoint("image")

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_get_session_endpoint_unavailable(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = None
        get_session_mock.return_value = session_mock
        image_endpoint = self.cloud.get_session_endpoint("image")
        self.assertIsNone(image_endpoint)

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_get_session_endpoint_identity(self, get_session_mock):
        session_mock = mock.Mock()
        get_session_mock.return_value = session_mock
        self.cloud.get_session_endpoint('identity')
        session_mock.get_endpoint.assert_called_with(
            interface=ksa_plugin.AUTH_INTERFACE)

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_has_service_no(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = None
        get_session_mock.return_value = session_mock
        self.assertFalse(self.cloud.has_service("image"))

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    def test_has_service_yes(self, get_session_mock):
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = 'http://fake.url'
        get_session_mock.return_value = session_mock
        self.assertTrue(self.cloud.has_service("image"))

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_list_hypervisors(self, mock_nova):
        '''This test verifies that calling list_hypervisors results in a call
        to nova client.'''
        mock_nova.hypervisors.list.return_value = [
            fakes.FakeHypervisor('1', 'testserver1'),
            fakes.FakeHypervisor('2', 'testserver2'),
        ]

        r = self.cloud.list_hypervisors()
        mock_nova.hypervisors.list.assert_called_once_with()
        self.assertEquals(2, len(r))
        self.assertEquals('testserver1', r[0]['hypervisor_hostname'])
        self.assertEquals('testserver2', r[1]['hypervisor_hostname'])



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import inspect
import munch
import netifaces
import re
import six
import time

from decorator import decorator
from neutronclient.common import exceptions as neutron_exc

from shade import _log
from shade import exc
from shade import meta


log = _log.setup_logging(__name__)


_decorated_methods = []


def _iterate_timeout(timeout, message, wait=2):
    """Iterate and raise an exception on timeout.

    This is a generator that will continually yield and sleep for
    wait seconds, and if the timeout is reached, will raise an exception
    with <message>.

    """

    try:
        wait = float(wait)
    except ValueError:
        raise exc.OpenStackCloudException(
            "Wait value must be an int or float value. {wait} given"
            " instead".format(wait=wait))

    start = time.time()
    count = 0
    while (timeout is None) or (time.time() < start + timeout):
        count += 1
        yield count
        log.debug('Waiting {wait} seconds'.format(wait=wait))
        time.sleep(wait)
    raise exc.OpenStackCloudTimeout(message)


def _filter_list(data, name_or_id, filters):
    """Filter a list by name/ID and arbitrary meta data.

    :param list data:
        The list of dictionary data to filter. It is expected that
        each dictionary contains an 'id' and 'name'
        key if a value for name_or_id is given.
    :param string name_or_id:
        The name or ID of the entity being filtered.
    :param dict filters:
        A dictionary of meta data to use for further filtering. Elements
        of this dictionary may, themselves, be dictionaries. Example::

            {
              'last_name': 'Smith',
              'other': {
                  'gender': 'Female'
              }
            }
    """
    if name_or_id:
        identifier_matches = []
        for e in data:
            e_id = str(e.get('id', None))
            e_name = e.get('name', None)
            if str(name_or_id) in (e_id, e_name):
                identifier_matches.append(e)
        data = identifier_matches

    if not filters:
        return data

    def _dict_filter(f, d):
        if not d:
            return False
        for key in f.keys():
            if isinstance(f[key], dict):
                if not _dict_filter(f[key], d.get(key, None)):
                    return False
            elif d.get(key, None) != f[key]:
                return False
        return True

    filtered = []
    for e in data:
        filtered.append(e)
        for key in filters.keys():
            if isinstance(filters[key], dict):
                if not _dict_filter(filters[key], e.get(key, None)):
                    filtered.pop()
                    break
            elif e.get(key, None) != filters[key]:
                filtered.pop()
                break
    return filtered


def _get_entity(func, name_or_id, filters):
    """Return a single entity from the list returned by a given method.

    :param callable func:
        A function that takes `name_or_id` and `filters` as parameters
        and returns a list of entities to filter.
    :param string name_or_id:
        The name or ID of the entity being filtered or a dict
    :param dict filters:
        A dictionary of meta data to use for further filtering.
    """
    # Sometimes in the control flow of shade, we already have an object
    # fetched. Rather than then needing to pull the name or id out of that
    # object, pass it in here and rely on caching to prevent us from making
    # an additional call, it's simple enough to test to see if we got an
    # object and just short-circuit return it.
    if hasattr(name_or_id, 'id'):
        return name_or_id
    entities = func(name_or_id, filters)
    if not entities:
        return None
    if len(entities) > 1:
        raise exc.OpenStackCloudException(
            "Multiple matches found for %s" % name_or_id)
    return entities[0]


def normalize_servers(servers, cloud_name, region_name):
    # Here instead of _utils because we need access to region and cloud
    # name from the cloud object
    ret = []
    for server in servers:
        ret.append(normalize_server(server, cloud_name, region_name))
    return ret


def normalize_server(server, cloud_name, region_name):
    server.pop('links', None)
    server['flavor'].pop('links', None)
    # OpenStack can return image as a string when you've booted
    # from volume
    if str(server['image']) != server['image']:
        server['image'].pop('links', None)

    server['region'] = region_name
    server['cloud'] = cloud_name

    az = server.get('OS-EXT-AZ:availability_zone', None)
    if az:
        server['az'] = az

    # Ensure volumes is always in the server dict, even if empty
    server['volumes'] = []

    return server


def normalize_keystone_services(services):
    """Normalize the structure of keystone services

    In keystone v2, there is a field called "service_type". In v3, it's
    "type". Just make the returned dict have both.

    :param list services: A list of keystone service dicts

    :returns: A list of normalized dicts.
    """
    ret = []
    for service in services:
        service_type = service.get('type', service.get('service_type'))
        new_service = {
            'id': service['id'],
            'name': service['name'],
            'description': service.get('description', None),
            'type': service_type,
            'service_type': service_type,
            'enabled': service['enabled']
        }
        ret.append(new_service)
    return meta.obj_list_to_dict(ret)


def normalize_nova_secgroups(groups):
    """Normalize the structure of nova security groups

    This makes security group dicts, as returned from nova, look like the
    security group dicts as returned from neutron. This does not make them
    look exactly the same, but it's pretty close.

    :param list groups: A list of security group dicts.

    :returns: A list of normalized dicts.
    """
    ret = [{'id': g['id'],
            'name': g['name'],
            'description': g['description'],
            'security_group_rules': normalize_nova_secgroup_rules(g['rules'])
            } for g in groups]
    return meta.obj_list_to_dict(ret)


def normalize_nova_secgroup_rules(rules):
    """Normalize the structure of nova security group rules

    Note that nova uses -1 for non-specific port values, but neutron
    represents these with None.

    :param list rules: A list of security group rule dicts.

    :returns: A list of normalized dicts.
    """
    ret = [{'id': r['id'],
            'direction': 'ingress',
            'ethertype': 'IPv4',
            'port_range_min':
                None if r['from_port'] == -1 else r['from_port'],
            'port_range_max':
                None if r['to_port'] == -1 else r['to_port'],
            'protocol': r['ip_protocol'],
            'remote_ip_prefix': r['ip_range'].get('cidr', None),
            'security_group_id': r['parent_group_id']
            } for r in rules]
    return meta.obj_list_to_dict(ret)


def normalize_nova_floating_ips(ips):
    """Normalize the structure of Neutron floating IPs

    Unfortunately, not all the Neutron floating_ip attributes are available
    with Nova and not all Nova floating_ip attributes are available with
    Neutron.
    This function extract attributes that are common to Nova and Neutron
    floating IP resource.
    If the whole structure is needed inside shade, shade provides private
    methods that returns "original" objects (e.g. _nova_allocate_floating_ip)

    :param list ips: A list of Nova floating IPs.

    :returns:
        A list of normalized dicts with the following attributes::

        [
          {
            "id": "this-is-a-floating-ip-id",
            "fixed_ip_address": "192.0.2.10",
            "floating_ip_address": "198.51.100.10",
            "network": "this-is-a-net-or-pool-id",
            "attached": True,
            "status": "ACTIVE"
          }, ...
        ]

    """
    ret = [dict(
        id=ip['id'],
        fixed_ip_address=ip.get('fixed_ip'),
        floating_ip_address=ip['ip'],
        network=ip['pool'],
        attached=(ip.get('instance_id') is not None and
                  ip.get('instance_id') != ''),
        status='ACTIVE'  # In neutrons terms, Nova floating IPs are always
                         # ACTIVE
    ) for ip in ips]
    return meta.obj_list_to_dict(ret)


def normalize_neutron_floating_ips(ips):
    """Normalize the structure of Neutron floating IPs

    Unfortunately, not all the Neutron floating_ip attributes are available
    with Nova and not all Nova floating_ip attributes are available with
    Neutron.
    This function extract attributes that are common to Nova and Neutron
    floating IP resource.
    If the whole structure is needed inside shade, shade provides private
    methods that returns "original" objects (e.g.
    _neutron_allocate_floating_ip)

    :param list ips: A list of Neutron floating IPs.

    :returns:
        A list of normalized dicts with the following attributes::

        [
          {
            "id": "this-is-a-floating-ip-id",
            "fixed_ip_address": "192.0.2.10",
            "floating_ip_address": "198.51.100.10",
            "network": "this-is-a-net-or-pool-id",
            "attached": True,
            "status": "ACTIVE"
          }, ...
        ]

    """
    ret = [dict(
        id=ip['id'],
        fixed_ip_address=ip.get('fixed_ip_address'),
        floating_ip_address=ip['floating_ip_address'],
        network=ip['floating_network_id'],
        attached=(ip.get('port_id') is not None and
                  ip.get('port_id') != ''),
        status=ip['status']
    ) for ip in ips]
    return meta.obj_list_to_dict(ret)


def localhost_supports_ipv6():
    """Determine whether the local host supports IPv6

    We look for a default route that supports the IPv6 address family,
    and assume that if it is present, this host has globally routable
    IPv6 connectivity.
    """

    return netifaces.AF_INET6 in netifaces.gateways()['default']


def normalize_users(users):
    ret = [
        dict(
            id=user.get('id'),
            email=user.get('email'),
            name=user.get('name'),
            username=user.get('username'),
            default_project_id=user.get('default_project_id',
                                        user.get('tenantId')),
            domain_id=user.get('domain_id'),
            enabled=user.get('enabled'),
        ) for user in users
    ]
    return meta.obj_list_to_dict(ret)


def normalize_volumes(volumes):
    ret = []
    for vol in volumes:
        new_vol = vol.copy()
        name = vol.get('name', vol.get('display_name'))
        description = vol.get('description', vol.get('display_description'))
        new_vol['name'] = name
        new_vol['display_name'] = name
        new_vol['description'] = description
        new_vol['display_description'] = description
        # For some reason, cinder v1 uses strings for bools for these fields.
        # Cinder v2 uses booleans.
        for field in ('bootable', 'multiattach'):
            if field in new_vol and isinstance(new_vol[field],
                                               six.string_types):
                if new_vol[field] is not None:
                    if new_vol[field].lower() == 'true':
                        new_vol[field] = True
                    elif new_vol[field].lower() == 'false':
                        new_vol[field] = False
        ret.append(new_vol)
    return meta.obj_list_to_dict(ret)


def normalize_domains(domains):
    ret = [
        dict(
            id=domain.get('id'),
            name=domain.get('name'),
            description=domain.get('description'),
            enabled=domain.get('enabled'),
        ) for domain in domains
    ]
    return meta.obj_list_to_dict(ret)


def normalize_groups(domains):
    """Normalize Identity groups."""
    ret = [
        dict(
            id=domain.get('id'),
            name=domain.get('name'),
            description=domain.get('description'),
            domain_id=domain.get('domain_id'),
        ) for domain in domains
    ]
    return meta.obj_list_to_dict(ret)


def normalize_role_assignments(assignments):
    """Put role_assignments into a form that works with search/get interface.

    Role assignments have the structure::

        [
            {
                "role": {
                    "id": "--role-id--"
                },
                "scope": {
                    "domain": {
                        "id": "--domain-id--"
                    }
                },
                "user": {
                    "id": "--user-id--"
                }
            },
        ]

    Which is hard to work with in the rest of our interface. Map this to be::

        [
            {
                "id": "--role-id--",
                "domain": "--domain-id--",
                "user": "--user-id--",
            }
        ]

    Scope can be "domain" or "project" and "user" can also be "group".

    :param list assignments: A list of dictionaries of role assignments.

    :returns: A list of flattened/normalized role assignment dicts.
    """
    new_assignments = []
    for assignment in assignments:
        new_val = munch.Munch({'id': assignment['role']['id']})
        for scope in ('project', 'domain'):
            if scope in assignment['scope']:
                new_val[scope] = assignment['scope'][scope]['id']
        for assignee in ('user', 'group'):
            if assignee in assignment:
                new_val[assignee] = assignment[assignee]['id']
        new_assignments.append(new_val)
    return new_assignments


def normalize_roles(roles):
    """Normalize Identity roles."""
    ret = [
        dict(
            id=role.get('id'),
            name=role.get('name'),
        ) for role in roles
    ]
    return meta.obj_list_to_dict(ret)


def valid_kwargs(*valid_args):
    # This decorator checks if argument passed as **kwargs to a function are
    # present in valid_args.
    #
    # Typically, valid_kwargs is used when we want to distinguish between
    # None and omitted arguments and we still want to validate the argument
    # list.
    #
    # Example usage:
    #
    # @valid_kwargs('opt_arg1', 'opt_arg2')
    # def my_func(self, mandatory_arg1, mandatory_arg2, **kwargs):
    #   ...
    #
    @decorator
    def func_wrapper(func, *args, **kwargs):
        argspec = inspect.getargspec(func)
        for k in kwargs:
            if k not in argspec.args[1:] and k not in valid_args:
                raise TypeError(
                    "{f}() got an unexpected keyword argument "
                    "'{arg}'".format(f=inspect.stack()[1][3], arg=k))
        return func(*args, **kwargs)
    return func_wrapper


def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
    def _inner_cache_on_arguments(func):
        def _cache_decorator(obj, *args, **kwargs):
            the_method = obj._cache.cache_on_arguments(
                *cache_on_args, **cache_on_kwargs)(
                    func.__get__(obj, type(obj)))
            return the_method(*args, **kwargs)

        def invalidate(obj, *args, **kwargs):
            return obj._cache.cache_on_arguments()(func).invalidate(
                *args, **kwargs)

        _cache_decorator.invalidate = invalidate
        _cache_decorator.func = func
        _decorated_methods.append(func.__name__)

        return _cache_decorator
    return _inner_cache_on_arguments


@contextlib.contextmanager
def neutron_exceptions(error_message):
    try:
        yield
    except neutron_exc.NotFound as e:
        raise exc.OpenStackCloudResourceNotFound(
            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
    except neutron_exc.NeutronClientException as e:
        if e.status_code == 404:
            raise exc.OpenStackCloudURINotFound(
                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
        else:
            raise exc.OpenStackCloudException(
                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
    except Exception as e:
        raise exc.OpenStackCloudException(
            "{msg}: {exc}".format(msg=error_message, exc=str(e)))


@contextlib.contextmanager
def shade_exceptions(error_message=None):
    """Context manager for dealing with shade exceptions.

    :param string error_message: String to use for the exception message
        content on non-OpenStackCloudExceptions.

    Useful for avoiding wrapping shade OpenStackCloudException exceptions
    within themselves. Code called from within the context may throw such
    exceptions without having to catch and reraise them.

    Non-OpenStackCloudException exceptions thrown within the context will
    be wrapped and the exception message will be appended to the given error
    message.
    """
    try:
        yield
    except exc.OpenStackCloudException:
        raise
    except Exception as e:
        if error_message is None:
            error_message = str(e)
        raise exc.OpenStackCloudException(error_message)


def safe_dict_min(key, data):
    """Safely find the minimum for a given key in a list of dict objects.

    This will find the minimum integer value for specific dictionary key
    across a list of dictionaries. The values for the given key MUST be
    integers, or string representations of an integer.

    The dictionary key does not have to be present in all (or any)
    of the elements/dicts within the data set.

    :param string key: The dictionary key to search for the minimum value.
    :param list data: List of dicts to use for the data set.

    :returns: None if the field was not not found in any elements, or
        the minimum value for the field otherwise.
    """
    min_value = None
    for d in data:
        if (key in d) and (d[key] is not None):
            try:
                val = int(d[key])
            except ValueError:
                raise exc.OpenStackCloudException(
                    "Search for minimum value failed. "
                    "Value for {key} is not an integer: {value}".format(
                        key=key, value=d[key])
                )
            if (min_value is None) or (val < min_value):
                min_value = val
    return min_value


def safe_dict_max(key, data):
    """Safely find the maximum for a given key in a list of dict objects.

    This will find the maximum integer value for specific dictionary key
    across a list of dictionaries. The values for the given key MUST be
    integers, or string representations of an integer.

    The dictionary key does not have to be present in all (or any)
    of the elements/dicts within the data set.

    :param string key: The dictionary key to search for the maximum value.
    :param list data: List of dicts to use for the data set.

    :returns: None if the field was not not found in any elements, or
        the maximum value for the field otherwise.
    """
    max_value = None
    for d in data:
        if (key in d) and (d[key] is not None):
            try:
                val = int(d[key])
            except ValueError:
                raise exc.OpenStackCloudException(
                    "Search for maximum value failed. "
                    "Value for {key} is not an integer: {value}".format(
                        key=key, value=d[key])
                )
            if (max_value is None) or (val > max_value):
                max_value = val
    return max_value


def parse_range(value):
    """Parse a numerical range string.

    Breakdown a range expression into its operater and numerical parts.
    This expression must be a string. Valid values must be an integer string,
    optionally preceeded by one of the following operators::

        - "<"  : Less than
        - ">"  : Greater than
        - "<=" : Less than or equal to
        - ">=" : Greater than or equal to

    Some examples of valid values and function return values::

        - "1024"  : returns (None, 1024)
        - "<5"    : returns ("<", 5)
        - ">=100" : returns (">=", 100)

    :param string value: The range expression to be parsed.

    :returns: A tuple with the operator string (or None if no operator
        was given) and the integer value. None is returned if parsing failed.
    """
    if value is None:
        return None

    range_exp = re.match('(<|>|<=|>=){0,1}(\d+)$', value)
    if range_exp is None:
        return None

    op = range_exp.group(1)
    num = int(range_exp.group(2))
    return (op, num)


def range_filter(data, key, range_exp):
    """Filter a list by a single range expression.

    :param list data: List of dictionaries to be searched.
    :param string key: Key name to search within the data set.
    :param string range_exp: The expression describing the range of values.

    :returns: A list subset of the original data set.
    :raises: OpenStackCloudException on invalid range expressions.
    """
    filtered = []
    range_exp = str(range_exp).upper()

    if range_exp == "MIN":
        key_min = safe_dict_min(key, data)
        if key_min is None:
            return []
        for d in data:
            if int(d[key]) == key_min:
                filtered.append(d)
        return filtered
    elif range_exp == "MAX":
        key_max = safe_dict_max(key, data)
        if key_max is None:
            return []
        for d in data:
            if int(d[key]) == key_max:
                filtered.append(d)
        return filtered

    # Not looking for a min or max, so a range or exact value must
    # have been supplied.
    val_range = parse_range(range_exp)

    # If parsing the range fails, it must be a bad value.
    if val_range is None:
        raise exc.OpenStackCloudException(
            "Invalid range value: {value}".format(value=range_exp))

    op = val_range[0]
    if op:
        # Range matching
        for d in data:
            d_val = int(d[key])
            if op == '<':
                if d_val < val_range[1]:
                    filtered.append(d)
            elif op == '>':
                if d_val > val_range[1]:
                    filtered.append(d)
            elif op == '<=':
                if d_val <= val_range[1]:
                    filtered.append(d)
            elif op == '>=':
                if d_val >= val_range[1]:
                    filtered.append(d)
        return filtered
    else:
        # Exact number match
        for d in data:
            if int(d[key]) == val_range[1]:
                filtered.append(d)
        return filtered
